// Copyright (c) 2008  Future Technology Devices International Ltd.
//
// Module Name
//
//     MainForm.cs
//
// Abstract:
//
//    C++ console program, to test the FTCJTAG.DLL. The FTCJTAG.DLL is used to control the FT2232H dual hi-speed device
//    and the FT4232H quad hi-speed device to simulate the Joint Test Action Group(JTAG) synchronous protocol to
//    communicate with JTAG devices connected to FT2232H dual hi-speed devices and FT4232H quad hi-speed devices.
//
// Revision History:
//
//   30/07/08    Kirke Ashton    Created.
//
// FT2232HJTAGC++NETTestApp.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <windows.h>

#include "FTCJTAG.h"

int _tmain(int argc, _TCHAR* argv[])
{
  FTC_STATUS ftStatus = FTC_SUCCESS;
  DWORD dwNumHiSpeedDevices = 0;
  char szDeviceName[100];
  char szChannel[5];
  DWORD dwLocationID = 0;
  DWORD dwHiSpeedDeviceType = 0;
  DWORD dwHiSpeedDeviceIndex = 0;
  FTC_HANDLE ftHandle = 0;
  char szDeviceDetails[150];
  byte timerValue = 0;
  DWORD dwClockFrequencyHz = 0;
  FTC_INPUT_OUTPUT_PINS LowInputOutputPinsData;
  FTH_INPUT_OUTPUT_PINS HighInputOutputPinsData;
  FTC_LOW_HIGH_PINS LowPinsInputData;
  FTH_LOW_HIGH_PINS HighPinsInputData;
  FTC_CLOSE_FINAL_STATE_PINS CloseFinalStatePinsData;
  BOOL bPerformCommandSequence = FALSE;
  WriteDataByteBuffer WriteDataBuffer;
  ReadDataByteBuffer ReadDataBuffer;
  ReadCmdSequenceDataByteBuffer ReadCmdSequenceDataBuffer;
  DWORD dwNumBytesReturned = 0;
  DWORD dwLoopCntr = 0;
  char szDllVersion[10];
  char szStatusErrorMessage[100];
  char szErrorMessage[150];
  DWORD dwCharCntr = 0;
  int MsgBoxKeyPressed = 0;
  char szMismatchMessage[100];

  for (dwCharCntr = 0; (dwCharCntr < 100); dwCharCntr++)
    szMismatchMessage[dwCharCntr] = '\0';

  ftStatus = JTAG_GetDllVersion(szDllVersion, 10);
  MessageBox(NULL, szDllVersion, "FTCJTAG DLL Version", MB_OK);

  ftStatus = JTAG_GetNumHiSpeedDevices(&dwNumHiSpeedDevices);

  if ((ftStatus == FTC_SUCCESS) && (dwNumHiSpeedDevices > 0))
  {
    do
    {
      ftStatus = JTAG_GetHiSpeedDeviceNameLocIDChannel(dwHiSpeedDeviceIndex, szDeviceName, 100, &dwLocationID, szChannel, 5, &dwHiSpeedDeviceType);

      dwHiSpeedDeviceIndex = dwHiSpeedDeviceIndex + 1;
    }
    while ((ftStatus == FTC_SUCCESS) && (dwHiSpeedDeviceIndex < dwNumHiSpeedDevices) && (strcmp(szChannel, "B") != 0));

    if (ftStatus == FTC_SUCCESS)
    {
      if (strcmp(szChannel, "B") != 0)
        ftStatus = FTC_DEVICE_IN_USE;
    }

    if (ftStatus == FTC_SUCCESS) {
      ftStatus = JTAG_OpenHiSpeedDevice(szDeviceName, dwLocationID, szChannel, &ftHandle);

      if (ftStatus == FTC_SUCCESS) {
        ftStatus = JTAG_GetHiSpeedDeviceType(ftHandle, &dwHiSpeedDeviceType);

        if (ftStatus == FTC_SUCCESS) {
          strcpy_s(szDeviceDetails, "Type = ");

          if (dwHiSpeedDeviceType == FT4232H_DEVICE_TYPE)
            strcat_s(szDeviceDetails, "FT4232H");
          else if (dwHiSpeedDeviceType == FT2232H_DEVICE_TYPE)
            strcat_s(szDeviceDetails, "FT2232H");

          strcat_s(szDeviceDetails, ", Name = ");
          strcat_s(szDeviceDetails, szDeviceName);

          MessageBox(NULL, szDeviceDetails, "Hi Speed Device", MB_OK);
        }
      }
    }
  }

  if ((ftStatus == FTC_SUCCESS) && ((ftHandle != 0)))
  {
    ftStatus = JTAG_InitDevice(ftHandle, 0); //65535

    if (ftStatus == FTC_SUCCESS) {
      //ftStatus = JTAG_TurnOnAdaptiveClockingHiSpeedDevice(ftHandle);

      //ftStatus = JTAG_TurnOffAdaptiveClockingHiSpeedDevice(ftHandle);
    }

    if (ftStatus == FTC_SUCCESS) {
      if ((ftStatus = JTAG_GetDeviceLatencyTimer(ftHandle, &timerValue)) == FTC_SUCCESS) {
        if ((ftStatus = JTAG_SetDeviceLatencyTimer(ftHandle, 50)) == FTC_SUCCESS) {
          ftStatus = JTAG_GetDeviceLatencyTimer(ftHandle, &timerValue);

          ftStatus = JTAG_SetDeviceLatencyTimer(ftHandle, 16);

          ftStatus = JTAG_GetDeviceLatencyTimer(ftHandle, &timerValue);
        }
      }
    }

    if (ftStatus == FTC_SUCCESS)
    {
      if ((ftStatus = JTAG_GetHiSpeedDeviceClock(0, &dwClockFrequencyHz)) == FTC_SUCCESS)
      {
        if ((ftStatus = JTAG_TurnOnDivideByFiveClockingHiSpeedDevice(ftHandle)) == FTC_SUCCESS)
        {
          ftStatus = JTAG_GetHiSpeedDeviceClock(0, &dwClockFrequencyHz);

          if ((ftStatus = JTAG_SetClock(ftHandle, 0, &dwClockFrequencyHz)) == FTC_SUCCESS)
          {
            ftStatus = JTAG_TurnOffDivideByFiveClockingHiSpeedDevice(ftHandle);

            ftStatus = JTAG_SetClock(ftHandle, 1, &dwClockFrequencyHz);
          }
        }
      }
    }

    if (ftStatus == FTC_SUCCESS)
      ftStatus = JTAG_SetLoopback(ftHandle, false);

    if (ftStatus == FTC_SUCCESS)
    {
      LowInputOutputPinsData.bPin1InputOutputState = true;
      LowInputOutputPinsData.bPin2InputOutputState = true;
      LowInputOutputPinsData.bPin3InputOutputState = true;
      LowInputOutputPinsData.bPin4InputOutputState = true;
      LowInputOutputPinsData.bPin1LowHighState = true;
      LowInputOutputPinsData.bPin2LowHighState = true;
      LowInputOutputPinsData.bPin3LowHighState = true;
      LowInputOutputPinsData.bPin4LowHighState = true;

      if (dwHiSpeedDeviceType == FT2232H_DEVICE_TYPE) {
        HighInputOutputPinsData.bPin1InputOutputState = true;
        HighInputOutputPinsData.bPin2InputOutputState = true;
        HighInputOutputPinsData.bPin3InputOutputState = true;
        HighInputOutputPinsData.bPin4InputOutputState = true;
        HighInputOutputPinsData.bPin5InputOutputState = true;
        HighInputOutputPinsData.bPin6InputOutputState = true;
        HighInputOutputPinsData.bPin7InputOutputState = true;
        HighInputOutputPinsData.bPin8InputOutputState = true;
        HighInputOutputPinsData.bPin1LowHighState = true;
        HighInputOutputPinsData.bPin2LowHighState = false;
        HighInputOutputPinsData.bPin3LowHighState = false;
        HighInputOutputPinsData.bPin4LowHighState = true;
        HighInputOutputPinsData.bPin5LowHighState = true;
        HighInputOutputPinsData.bPin6LowHighState = false;
        HighInputOutputPinsData.bPin7LowHighState = false;
        HighInputOutputPinsData.bPin8LowHighState = true;

        ftStatus = JTAG_SetHiSpeedDeviceGPIOs(ftHandle, true, &LowInputOutputPinsData, true, &HighInputOutputPinsData);
      } else
        ftStatus = JTAG_SetHiSpeedDeviceGPIOs(ftHandle, true, &LowInputOutputPinsData, false, &HighInputOutputPinsData);

      // Sleep for 1 second
      Sleep(1000);

      if (ftStatus == FTC_SUCCESS) {
        if (dwHiSpeedDeviceType == FT2232H_DEVICE_TYPE)
          ftStatus = JTAG_GetHiSpeedDeviceGPIOs(ftHandle, true, &LowPinsInputData, true, &HighPinsInputData);
        else
          ftStatus = JTAG_GetHiSpeedDeviceGPIOs(ftHandle, true, &LowPinsInputData, false, &HighPinsInputData);
      }

      if (ftStatus == FTC_SUCCESS)
      {
        bPerformCommandSequence = FALSE;

        WriteDataBuffer[0] = '\x01';
        WriteDataBuffer[1] = '\x01';
        WriteDataBuffer[2] = '\x01';
        WriteDataBuffer[3] = '\x01';

        //if (bPerformCommandSequence)
        if (!bPerformCommandSequence)
        {
          do
          {
            if ((ftStatus = JTAG_Write(ftHandle, true, 32, &WriteDataBuffer, 4, RUN_TEST_IDLE_STATE)) == FTC_SUCCESS)
            {
              Sleep(10);

              if ((ftStatus = JTAG_Read(ftHandle, true, 32, &ReadDataBuffer, &dwNumBytesReturned, RUN_TEST_IDLE_STATE)) == FTC_SUCCESS) {
                if ((ftStatus = JTAG_WriteRead(ftHandle, true, 32, &WriteDataBuffer, 4, &ReadDataBuffer, &dwNumBytesReturned, RUN_TEST_IDLE_STATE)) == FTC_SUCCESS) {
                  if ((WriteDataBuffer[0] != ReadDataBuffer[0]) || (WriteDataBuffer[1] != ReadDataBuffer[1]) ||
                      (WriteDataBuffer[2] != ReadDataBuffer[2]) || (WriteDataBuffer[3] != ReadDataBuffer[3]))
                  {
                    byte mismatchValue = 0;

                    if (WriteDataBuffer[0] != ReadDataBuffer[0])
                      mismatchValue = ReadDataBuffer[0];
                    else if (WriteDataBuffer[1] != ReadDataBuffer[1])
                      mismatchValue = ReadDataBuffer[1];
                    else if (WriteDataBuffer[2] != ReadDataBuffer[2])
                      mismatchValue = ReadDataBuffer[2];
                    else if (WriteDataBuffer[3] != ReadDataBuffer[3])
                      mismatchValue = ReadDataBuffer[3];

                    sprintf(szMismatchMessage, "JTAG Write/Read mismatch, Loop = %i, Write Value = %X, Read Value = %X", dwLoopCntr, 1, mismatchValue);

                    MsgBoxKeyPressed = MessageBox(NULL, szMismatchMessage, "JTAG Read Error Message", MB_OKCANCEL);
                  }
                }
              }
            }

            dwLoopCntr++;
          }
          while ((dwLoopCntr < 10) && (ftStatus == FTC_SUCCESS) && (MsgBoxKeyPressed == IDOK));
        }
        else
        {
          if (dwNumHiSpeedDevices == 1)
          {
            JTAG_ClearCmdSequence();

            do
            {
              ftStatus = JTAG_AddWriteCmd(true, 32, &WriteDataBuffer, 4, RUN_TEST_IDLE_STATE);

              if (ftStatus == FTC_SUCCESS)
              {
                ftStatus = JTAG_AddReadCmd(true, 32, RUN_TEST_IDLE_STATE);

                if (ftStatus == FTC_SUCCESS)
                  ftStatus = JTAG_AddWriteReadCmd(true, 32, &WriteDataBuffer, 4, RUN_TEST_IDLE_STATE);
              }

              dwLoopCntr++;
            }
            while ((dwLoopCntr < 100) && (ftStatus == FTC_SUCCESS));
          }
          else
          {
            JTAG_ClearDeviceCmdSequence(ftHandle);

            do
            {
              ftStatus = JTAG_AddDeviceWriteCmd(ftHandle, true, 32, &WriteDataBuffer, 4, RUN_TEST_IDLE_STATE);

              if (ftStatus == FTC_SUCCESS)
              {
                ftStatus = JTAG_AddDeviceReadCmd(ftHandle, true, 32, RUN_TEST_IDLE_STATE);

                if (ftStatus == FTC_SUCCESS)
                  ftStatus = JTAG_AddDeviceWriteReadCmd(ftHandle, true, 32, &WriteDataBuffer, 4, RUN_TEST_IDLE_STATE);
              }

              dwLoopCntr++;
            }
            while ((dwLoopCntr < 1000) && (ftStatus == FTC_SUCCESS));
          }

          if (ftStatus == FTC_SUCCESS)
            ftStatus = JTAG_ExecuteCmdSequence(ftHandle, &ReadCmdSequenceDataBuffer, &dwNumBytesReturned);
        }
      }
    }
  }

  if (ftHandle != 0)
  {
    //CloseFinalStatePinsData.bTCKPinState = true;
    //CloseFinalStatePinsData.bTCKPinActiveState = true;
    //CloseFinalStatePinsData.bTDIPinState = true;
    //CloseFinalStatePinsData.bTDIPinActiveState = false;
    //CloseFinalStatePinsData.bTMSPinState = true;
    //CloseFinalStatePinsData.bTMSPinActiveState = false;

    //ftStatus = JTAG_CloseDevice(ftHandle, &CloseFinalStatePinsData);

    JTAG_Close(ftHandle);
    ftHandle = 0;
  }

  if ((ftStatus != FTC_SUCCESS) || (dwNumHiSpeedDevices == 0) || (strlen(szMismatchMessage) > 0))
  {
    if (ftStatus != FTC_SUCCESS)
    {
      sprintf_s(szErrorMessage, "Status Code(%u) - ", ftStatus);

      ftStatus = JTAG_GetErrorCodeString("EN", ftStatus, szStatusErrorMessage, 100);

      strcat_s(szErrorMessage, szStatusErrorMessage);

      MessageBox(NULL, szErrorMessage, "FTCJTAG DLL Error Status Message", MB_OK);
    } else {
      if (dwNumHiSpeedDevices == 0)
        strcpy_s(szErrorMessage, "There are no devices connected.");
      else
        strcpy_s(szErrorMessage, szMismatchMessage);

      MessageBox(NULL, szErrorMessage, "FTCJTAG DLL Error Message", MB_OK);
    }
  }
  else
  {
    //ftStatus = JTAG_GetDllVersion(szDllVersion, 10);

    //if (ftStatus == FTC_SUCCESS)
      //MessageBox(NULL, szDllVersion, "JTAG DLL Version", MB_OK);
    //else
    //{
      //ftStatus = JTAG_GetErrorCodeString("EN", ftStatus, szErrorMessage, 100);

      //MessageBox(NULL, szErrorMessage, "JTAG Error Status Message", MB_OK);
    //}

    strcpy_s(szErrorMessage, "Passed.");

    MessageBox(NULL, szErrorMessage, "FTCJTAG DLL Message", MB_OK);
  }

	return 0;
}
